@@ -6,10 +6,13 @@ use log::debug;
 use reqwest::header::{HeaderMap, HeaderName, HeaderValue};
 use serde::{Deserialize, Serialize};
 
+use crate::config::ConfigHash;
+
 #[derive(Debug, Serialize)]
 pub struct RequestCacheWrite<'a> {
     pub headers: Vec<(&'a str, &'a str)>,
     pub version: &'a str,
+    pub config_hash: ConfigHash<'a>,
 }
 
 #[derive(Debug, Deserialize)]
@@ -20,9 +23,19 @@ struct RequestCacheRead {
     /// May be missing if the cache was created by an older version.
     #[serde(default)]
     version: Option<String>,
+    /// Hash of the config
+    ///
+    /// Used as cache buster when config changes.
+    ///
+    /// May be missing if the cache was created by an older version.
+    #[serde(default)]
+    config_hash: Option<String>,
 }
 
-pub fn deserialise_cached_headers(path: &Path) -> Option<HeaderMap<HeaderValue>> {
+pub fn deserialise_cached_headers(
+    path: &Path,
+    config_hash: ConfigHash<'_>,
+) -> Option<HeaderMap<HeaderValue>> {
     let raw = fs::read(path).ok()?;
     let cache: RequestCacheRead = toml::from_slice(&raw).ok()?;
 
@@ -34,6 +47,14 @@ pub fn deserialise_cached_headers(path: &Path) -> Option<HeaderMap<HeaderValue>>
             path.display()
         );
         return None;
+    } else if cache.config_hash.as_deref() != Some(config_hash.0) {
+        debug!(
+            "cache config hash mismatch ({:?}) != ({:?}), ignoring cache at: {}",
+            cache.config_hash,
+            config_hash,
+            path.display()
+        );
+        return None;
     }
 
     debug!("using cache at: {}", path.display());
