@@ -1,4 +1,4 @@
-use std::mem;
+use std::{fs, mem};
 
 use basic_toml as toml;
 use kuchiki::traits::TendrilSink;
@@ -6,16 +6,19 @@ use kuchiki::{ElementData, NodeDataRef, NodeRef};
 use log::{debug, error, info, warn};
 use mime_guess::mime;
 use reqwest::header::HeaderMap;
-use reqwest::{Client, RequestBuilder, StatusCode};
+use reqwest::{RequestBuilder, StatusCode};
 use rss::{Channel, ChannelBuilder, EnclosureBuilder, GuidBuilder, Item, ItemBuilder};
-use simple_eyre::eyre::{self, eyre, WrapErr};
+use simple_eyre::eyre::{self, bail, eyre, WrapErr};
 use time::format_description::well_known::Rfc2822;
 use time::OffsetDateTime;
+use tokio::task;
 use url::Url;
 
 use crate::cache::RequestCacheWrite;
 use crate::config::{ChannelConfig, ConfigHash, DateConfig, FeedConfig};
+use crate::Client;
 
+#[derive(Debug)]
 pub enum ProcessResult {
     NotModified,
     Ok {
@@ -24,6 +27,14 @@ pub enum ProcessResult {
     },
 }
 
+pub enum FetchResult {
+    NotModified,
+    Ok {
+        html: String,
+        headers: Option<String>,
+    },
+}
+
 pub async fn process_feed(
     client: &Client,
     channel_config: &ChannelConfig,
@@ -36,61 +47,15 @@ pub async fn process_feed(
         .url
         .parse()
         .wrap_err_with(|| format!("unable to parse {} as a URL", config.url))?;
-    let req = add_headers(
-        client.get(url.clone()),
-        cached_headers,
-        &channel_config.user_agent,
-    );
 
-    let resp = req
-        .send()
-        .await
-        .wrap_err_with(|| format!("unable to fetch {}", url))?;
-
-    // Check response
-    let status = resp.status();
-    if status == StatusCode::NOT_MODIFIED {
-        // Cache hit, nothing to do
-        info!("{} is unmodified", url);
-        return Ok(ProcessResult::NotModified);
-    }
-
-    if !status.is_success() {
-        return Err(eyre!(
-            "failed to fetch {}: {} {}",
-            config.url,
-            status.as_str(),
-            status.canonical_reason().unwrap_or("Unknown Status")
-        ));
-    }
-
-    if config.link.is_none() {
-        info!(
-            "no explicit link selector provided, falling back to heading selector: {:?}",
-            config.heading
-        );
-    }
+    let (html, serialised_headers) =
+        match fetch_webpage(client, &url, cached_headers, channel_config, config_hash).await? {
+            FetchResult::Ok { html, headers } => (html, headers),
+            FetchResult::NotModified => return Ok(ProcessResult::NotModified),
+        };
 
     let link_selector = config.link.as_ref().unwrap_or(&config.heading);
 
-    // Collect the headers for later
-    let headers: Vec<_> = resp
-        .headers()
-        .iter()
-        .filter_map(|(name, value)| value.to_str().ok().map(|val| (name.as_str(), val)))
-        .collect();
-    let map = RequestCacheWrite {
-        headers,
-        version: crate::version(),
-        config_hash,
-    };
-    let serialised_headers = toml::to_string(&map)
-        .map_err(|err| warn!("unable to serialise headers: {}", err))
-        .ok();
-
-    // Read body
-    let html = resp.text().await.wrap_err("unable to read response body")?;
-
     let doc = kuchiki::parse_html().one(html);
     let base_url = Url::options().base_url(Some(&url));
     rewrite_urls(&doc, &base_url)?;
@@ -125,6 +90,109 @@ pub async fn process_feed(
     })
 }
 
+async fn fetch_webpage(
+    client: &Client,
+    url: &Url,
+    cached_headers: &Option<HeaderMap>,
+    channel_config: &ChannelConfig,
+    config_hash: ConfigHash<'_>,
+) -> eyre::Result<FetchResult> {
+    if url.scheme() == "file" {
+        if client.file_urls {
+            fetch_webpage_local(url).await
+        } else {
+            bail!("unable to fetch: {url} as file URLs are not enabled in config")
+        }
+    } else {
+        fetch_webpage_http(client, url, cached_headers, channel_config, config_hash).await
+    }
+}
+
+async fn fetch_webpage_http(
+    client: &Client,
+    url: &Url,
+    cached_headers: &Option<HeaderMap>,
+    channel_config: &ChannelConfig,
+    config_hash: ConfigHash<'_>,
+) -> eyre::Result<FetchResult> {
+    let config = &channel_config.config;
+
+    let req = add_headers(
+        client.http.get(url.clone()),
+        cached_headers,
+        &channel_config.user_agent,
+    );
+
+    let resp = req
+        .send()
+        .await
+        .wrap_err_with(|| format!("unable to fetch {}", url))?;
+
+    // Check response
+    let status = resp.status();
+    if status == StatusCode::NOT_MODIFIED {
+        // Cache hit, nothing to do
+        info!("{} is unmodified", url);
+        return Ok(FetchResult::NotModified);
+    }
+
+    if !status.is_success() {
+        return Err(eyre!(
+            "failed to fetch {}: {} {}",
+            config.url,
+            status.as_str(),
+            status.canonical_reason().unwrap_or("Unknown Status")
+        ));
+    }
+
+    if config.link.is_none() {
+        info!(
+            "no explicit link selector provided, falling back to heading selector: {:?}",
+            config.heading
+        );
+    }
+
+    // Collect the headers for later
+    let headers: Vec<_> = resp
+        .headers()
+        .iter()
+        .filter_map(|(name, value)| value.to_str().ok().map(|val| (name.as_str(), val)))
+        .collect();
+    let map = RequestCacheWrite {
+        headers,
+        version: crate::version(),
+        config_hash,
+    };
+    let serialised_headers = toml::to_string(&map)
+        .map_err(|err| warn!("unable to serialise headers: {}", err))
+        .ok();
+
+    // Read body
+    let html = resp.text().await.wrap_err("unable to read response body")?;
+
+    Ok(FetchResult::Ok {
+        html,
+        headers: serialised_headers,
+    })
+}
+
+async fn fetch_webpage_local(url: &Url) -> eyre::Result<FetchResult> {
+    let path = url
+        .to_file_path()
+        .map_err(|()| eyre!("unable to extract path from: {}", url))?;
+    debug!("read {}", path.display());
+    let html = task::spawn_blocking(move || {
+        fs::read_to_string(&path).wrap_err_with(|| format!("error reading {}", path.display()))
+    })
+    .await
+    .wrap_err_with(|| format!("error joining task for {url}"))??;
+
+    Ok(FetchResult::Ok {
+        html,
+        headers: None,
+    })
+}
+
 fn process_item(
     config: &FeedConfig,
     item: NodeDataRef<ElementData>,
@@ -327,8 +395,33 @@ fn extract_description(
 
 #[cfg(test)]
 mod tests {
+    use std::path::{Path, PathBuf};
+    use std::{env, process};
+
+    use reqwest::Client as HttpClient;
+
     use super::*;
 
+    const HTML: &str = include_str!("../tests/local.html");
+
+    struct RmOnDrop(PathBuf);
+
+    impl RmOnDrop {
+        fn new(path: PathBuf) -> Self {
+            RmOnDrop(path)
+        }
+
+        fn path(&self) -> &Path {
+            &self.0
+        }
+    }
+
+    impl Drop for RmOnDrop {
+        fn drop(&mut self) {
+            let _ = fs::remove_file(&self.0);
+        }
+    }
+
     fn test_config() -> FeedConfig {
         FeedConfig {
             url: String::new(),
@@ -399,4 +492,93 @@ mod tests {
         // Items come out in the order of the selector array
         assert_eq!(description, "<span>two</span><p>one</p>");
     }
+
+    #[test]
+    fn test_process_local_html() {
+        let html_file_name = format!("rsspls.local.{}.html", process::id());
+        let local_html = RmOnDrop::new(env::temp_dir().join(&html_file_name));
+        fs::write(local_html.path(), HTML.as_bytes()).expect("unable to write test HTML");
+
+        let url = Url::from_file_path(local_html.path())
+            .expect("unable to construct file URL for test HTML");
+
+        let client = Client {
+            file_urls: true,
+            http: HttpClient::new(),
+        };
+
+        let config = FeedConfig {
+            url: url.to_string(),
+            item: "nav a".to_string(),
+            heading: "a".to_string(),
+            ..test_config()
+        };
+        let channel_config = ChannelConfig {
+            title: "Local Site".to_string(),
+            filename: Path::new(&html_file_name)
+                .with_extension("rss")
+                .to_string_lossy()
+                .into_owned(),
+            user_agent: None,
+            config,
+        };
+        let config_hash = ConfigHash(&html_file_name);
+
+        let runtime = tokio::runtime::Builder::new_current_thread()
+            .build()
+            .unwrap();
+        let res = runtime
+            .block_on(process_feed(&client, &channel_config, config_hash, &None))
+            .expect("unable to process local feed");
+
+        let ProcessResult::Ok { channel, .. } = res else {
+            panic!("expected ProcessResult::Ok but got: {:?}", res)
+        };
+
+        assert_eq!(channel.items().len(), 5);
+        assert_eq!(channel.items()[0].title, Some("Install".to_string()));
+    }
+
+    #[test]
+    fn test_process_local_files_disabled() {
+        let html_file_name = "rsspls.local.html";
+        let local_html = env::temp_dir().join(&html_file_name);
+        let url =
+            Url::from_file_path(&local_html).expect("unable to construct file URL for test HTML");
+
+        let client = Client {
+            file_urls: false,
+            http: HttpClient::new(),
+        };
+
+        let config = FeedConfig {
+            url: url.to_string(),
+            item: "nav a".to_string(),
+            heading: "a".to_string(),
+            ..test_config()
+        };
+        let channel_config = ChannelConfig {
+            title: "Local Site".to_string(),
+            filename: Path::new(&html_file_name)
+                .with_extension("rss")
+                .to_string_lossy()
+                .into_owned(),
+            user_agent: None,
+            config,
+        };
+        let config_hash = ConfigHash(&html_file_name);
+
+        let runtime = tokio::runtime::Builder::new_current_thread()
+            .build()
+            .unwrap();
+        let res = runtime.block_on(process_feed(&client, &channel_config, config_hash, &None));
+
+        let Err(err) = res else {
+            panic!("expected error, got: {:?}", res)
+        };
+
+        assert!(err
+            .to_string()
+            .contains("file URLs are not enabled in config"));
+    }
 }
